{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014-2017
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
module F4L.FileTree (
    FileTree(..)
  , allFiles
  , empty
  , files
  , findFileNode
  , findNode
  , subDirs
  , subFiles
  ) where

import F4L.Utils

import qualified Data.Map.Strict as SM
import System.FilePath( splitDirectories )


data FileTree a = File a
                | Dir ( SM.Map ShortPath ( FileTree a ) )


empty :: FileTree a
empty = Dir SM.empty


files :: FileTree a -> [a]
files ( File fid ) = return fid
files ( Dir d ) = concatMap files $ SM.elems d


subDirs :: FileTree a -> [FilePath]
subDirs ( Dir st ) = do
  (p, v) <- SM.toList st
  case v of
    Dir _ -> return $ shortToPath p
    _ -> []
subDirs _ = []


allFiles :: FileTree a -> [([FilePath], a)]
allFiles ( File f ) = return ([], f)
allFiles ( Dir st ) = do 
  ( p, node ) <- SM.toList st
  ( path, fid ) <- allFiles node
  return ( shortToPath p : path, fid )


findNode :: [FilePath] -> FileTree a -> Maybe ( FileTree a )
findNode [] st = Just st
findNode _ ( File _ ) = Nothing 
findNode ( x : xs ) ( Dir st ) = SM.lookup ( pathToShort x ) st >>= findNode xs


findFileNode :: FilePath -> FileTree a -> Maybe a
findFileNode path st =
  case findNode ( splitDirectories path ) st of
    Just ( File x ) -> Just x
    _ -> Nothing


-- | list of (relative subpaths, fileid) - pre is omitted
subFiles :: [FilePath] -> FileTree a -> [([FilePath], a)]
subFiles xs st = case findNode xs st of
  Nothing -> []
  Just n -> allFiles n
